@@ -14,6 +14,7 @@ from ..core.utils import (
     FrozenDict,
     close_on_error,
     is_remote_uri,
+    module_available,
     try_read_magic_number_from_path,
 )
 from ..core.variable import Variable
@@ -31,17 +32,6 @@ from .locks import HDF5_LOCK, NETCDFC_LOCK, combine_locks, ensure_lock, get_writ
 from .netcdf3 import encode_nc3_attr_value, encode_nc3_variable
 from .store import StoreBackendEntrypoint
 
-try:
-    import netCDF4
-
-    has_netcdf4 = True
-except ImportError:
-    # Except a base ImportError (not ModuleNotFoundError) to catch usecases
-    # where errors have mismatched versions of c-dependencies. This can happen
-    # when developers are making changes them.
-    has_netcdf4 = False
-
-
 # This lookup table maps from dtype.byteorder to a readable endian
 # string used by netCDF4.
 _endian_lookup = {"=": "native", ">": "big", "<": "little", "|": "native"}
@@ -242,11 +232,11 @@ def _extract_nc4_variable_encoding(
         "shuffle",
         "_FillValue",
         "dtype",
+        "compression",
     }
     if lsd_okay:
         valid_encodings.add("least_significant_digit")
     if h5py_okay:
-        valid_encodings.add("compression")
         valid_encodings.add("compression_opts")
 
     if not raise_on_invalid and encoding.get("chunksizes") is not None:
@@ -313,6 +303,7 @@ class NetCDF4DataStore(WritableCFDataStore):
     def __init__(
         self, manager, group=None, mode=None, lock=NETCDF4_PYTHON_LOCK, autoclose=False
     ):
+        import netCDF4
 
         if isinstance(manager, netCDF4.Dataset):
             if group is None:
@@ -349,6 +340,7 @@ class NetCDF4DataStore(WritableCFDataStore):
         lock_maker=None,
         autoclose=False,
     ):
+        import netCDF4
 
         if isinstance(filename, os.PathLike):
             filename = os.fspath(filename)
@@ -516,7 +508,32 @@ class NetCDF4DataStore(WritableCFDataStore):
 
 
 class NetCDF4BackendEntrypoint(BackendEntrypoint):
-    available = has_netcdf4
+    """
+    Backend for netCDF files based on the netCDF4 package.
+
+    It can open ".nc", ".nc4", ".cdf" files and will be choosen
+    as default for these files.
+
+    Additionally it can open valid HDF5 files, see
+    https://h5netcdf.org/#invalid-netcdf-files for more info.
+    It will not be detected as valid backend for such files, so make
+    sure to specify ``engine="netcdf4"`` in ``open_dataset``.
+
+    For more information about the underlying library, visit:
+    https://unidata.github.io/netcdf4-python
+
+    See Also
+    --------
+    backends.NetCDF4DataStore
+    backends.H5netcdfBackendEntrypoint
+    backends.ScipyBackendEntrypoint
+    """
+
+    available = module_available("netCDF4")
+    description = (
+        "Open netCDF (.nc, .nc4 and .cdf) and most HDF5 files using netCDF4 in Xarray"
+    )
+    url = "https://docs.xarray.dev/en/stable/generated/xarray.backends.NetCDF4BackendEntrypoint.html"
 
     def guess_can_open(self, filename_or_obj):
         if isinstance(filename_or_obj, str) and is_remote_uri(filename_or_obj):
