@@ -581,7 +581,18 @@ def _encode_datetime_with_cftime(dates, units, calendar):
         dates = dates.astype("M8[us]").astype(datetime)
 
     def encode_datetime(d):
-        return np.nan if d is None else cftime.date2num(d, units, calendar)
+        # Since netCDF files do not support storing float128 values, we ensure
+        # that float64 values are used by setting longdouble=False in num2date.
+        # This try except logic can be removed when xarray's minimum version of
+        # cftime is at least 1.6.2.
+        try:
+            return (
+                np.nan
+                if d is None
+                else cftime.date2num(d, units, calendar, longdouble=False)
+            )
+        except TypeError:
+            return np.nan if d is None else cftime.date2num(d, units, calendar)
 
     return np.array([encode_datetime(d) for d in dates.ravel()]).reshape(dates.shape)
 
