@@ -21,7 +21,6 @@ import numpy as np
 import pandas as pd
 
 from . import dtypes, duck_array_ops, formatting, formatting_html, ops
-from .npcompat import DTypeLike, DTypeLikeSave
 from .options import OPTIONS, _get_keep_attrs
 from .pycompat import is_duck_dask_array
 from .utils import Frozen, either_dict_or_kwargs, is_scalar
@@ -38,14 +37,24 @@ ALL_DIMS = ...
 if TYPE_CHECKING:
     import datetime
 
+    from numpy.typing import DTypeLike
+
     from .dataarray import DataArray
     from .dataset import Dataset
     from .indexes import Index
     from .resample import Resample
     from .rolling_exp import RollingExp
-    from .types import ScalarOrArray, SideOptions, T_DataWithCoords
+    from .types import (
+        DatetimeLike,
+        DTypeLikeSave,
+        ScalarOrArray,
+        SideOptions,
+        T_DataWithCoords,
+    )
     from .variable import Variable
 
+    DTypeMaybeMapping = Union[DTypeLikeSave, Mapping[Any, DTypeLikeSave]]
+
 
 T_Resample = TypeVar("T_Resample", bound="Resample")
 C = TypeVar("C")
@@ -163,9 +172,22 @@ class AbstractArray:
             return f"<pre>{escape(repr(self))}</pre>"
         return formatting_html.array_repr(self)
 
-    def __format__(self: Any, format_spec: str) -> str:
-        # we use numpy: scalars will print fine and arrays will raise
-        return self.values.__format__(format_spec)
+    def __format__(self: Any, format_spec: str = "") -> str:
+        if format_spec != "":
+            if self.shape == ():
+                # Scalar values might be ok use format_spec with instead of repr:
+                return self.data.__format__(format_spec)
+            else:
+                # TODO: If it's an array the formatting.array_repr(self) should
+                # take format_spec as an input. If we'd only use self.data we
+                # lose all the information about coords for example which is
+                # important information:
+                raise NotImplementedError(
+                    "Using format_spec is only supported"
+                    f" when shape is (). Got shape = {self.shape}."
+                )
+        else:
+            return self.__repr__()
 
     def _iter(self: Any) -> Iterator[Any]:
         for n in range(len(self)):
@@ -585,6 +607,7 @@ class DataWithCoords(AttrAccessMixin):
         --------
         Dataset.assign
         Dataset.swap_dims
+        Dataset.set_coords
         """
         coords_combined = either_dict_or_kwargs(coords, coords_kwargs, "assign_coords")
         data = self.copy(deep=False)
@@ -753,7 +776,7 @@ class DataWithCoords(AttrAccessMixin):
 
     def rolling_exp(
         self: T_DataWithCoords,
-        window: Mapping[Any, int] = None,
+        window: Mapping[Any, int] | None = None,
         window_type: str = "span",
         **window_kwargs,
     ) -> RollingExp[T_DataWithCoords]:
@@ -800,7 +823,9 @@ class DataWithCoords(AttrAccessMixin):
         skipna: bool | None,
         closed: SideOptions | None,
         label: SideOptions | None,
-        base: int,
+        base: int | None,
+        offset: pd.Timedelta | datetime.timedelta | str | None,
+        origin: str | DatetimeLike,
         keep_attrs: bool | None,
         loffset: datetime.timedelta | str | None,
         restore_coord_dims: bool | None,
@@ -828,6 +853,18 @@ class DataWithCoords(AttrAccessMixin):
             For frequencies that evenly subdivide 1 day, the "origin" of the
             aggregated intervals. For example, for "24H" frequency, base could
             range from 0 through 23.
+        origin : {'epoch', 'start', 'start_day', 'end', 'end_day'}, pd.Timestamp, datetime.datetime, np.datetime64, or cftime.datetime, default 'start_day'
+            The datetime on which to adjust the grouping. The timezone of origin
+            must match the timezone of the index.
+
+            If a datetime is not used, these values are also supported:
+            - 'epoch': `origin` is 1970-01-01
+            - 'start': `origin` is the first value of the timeseries
+            - 'start_day': `origin` is the first day at midnight of the timeseries
+            - 'end': `origin` is the last value of the timeseries
+            - 'end_day': `origin` is the ceiling midnight of the last day
+        offset : pd.Timedelta, datetime.timedelta, or str, default is None
+            An offset timedelta added to the origin.
         loffset : timedelta or str, optional
             Offset used to adjust the resampled time labels. Some pandas date
             offset strings are supported.
@@ -943,10 +980,24 @@ class DataWithCoords(AttrAccessMixin):
             if isinstance(self._indexes[dim_name].to_pandas_index(), CFTimeIndex):
                 from .resample_cftime import CFTimeGrouper
 
-                grouper = CFTimeGrouper(freq, closed, label, base, loffset)
+                grouper = CFTimeGrouper(
+                    freq=freq,
+                    closed=closed,
+                    label=label,
+                    base=base,
+                    loffset=loffset,
+                    origin=origin,
+                    offset=offset,
+                )
             else:
                 grouper = pd.Grouper(
-                    freq=freq, closed=closed, label=label, base=base, loffset=loffset
+                    freq=freq,
+                    closed=closed,
+                    label=label,
+                    base=base,
+                    offset=offset,
+                    origin=origin,
+                    loffset=loffset,
                 )
         group = DataArray(
             dim_coord, coords=dim_coord.coords, dims=dim_coord.dims, name=RESAMPLE_DIM
@@ -1063,7 +1114,9 @@ class DataWithCoords(AttrAccessMixin):
                 return cond.any(dim=(d for d in cond.dims if d != dim))
 
             def _dataset_indexer(dim: Hashable) -> DataArray:
-                cond_wdim = cond.drop(var for var in cond if dim not in cond[var].dims)
+                cond_wdim = cond.drop_vars(
+                    var for var in cond if dim not in cond[var].dims
+                )
                 keepany = cond_wdim.any(dim=(d for d in cond.dims.keys() if d != dim))
                 return keepany.to_array().any("variable")
 
@@ -1329,9 +1382,6 @@ class DataWithCoords(AttrAccessMixin):
         raise NotImplementedError()
 
 
-DTypeMaybeMapping = Union[DTypeLikeSave, Mapping[Any, DTypeLikeSave]]
-
-
 @overload
 def full_like(
     other: DataArray, fill_value: Any, dtype: DTypeLikeSave = None
@@ -1745,7 +1795,7 @@ def _contains_cftime_datetimes(array) -> bool:
         return False
     else:
         if array.dtype == np.dtype("O") and array.size > 0:
-            sample = array.ravel()[0]
+            sample = np.asarray(array).flat[0]
             if is_duck_dask_array(sample):
                 sample = sample.compute()
                 if isinstance(sample, np.ndarray):
