@@ -17,7 +17,7 @@ from packaging.version import Version
 from . import duck_array_ops
 from .nputils import NumpyVIndexAdapter
 from .options import OPTIONS
-from .pycompat import dask_version, integer_types, is_duck_dask_array, sparse_array_type
+from .pycompat import array_type, integer_types, is_duck_dask_array, mod_version
 from .types import T_Xarray
 from .utils import (
     NDArrayMixin,
@@ -180,7 +180,7 @@ def map_index_queries(
             # forward dimension indexers with no index/coordinate
             results.append(IndexSelResult(labels))
         else:
-            results.append(index.sel(labels, **options))  # type: ignore[call-arg]
+            results.append(index.sel(labels, **options))
 
     merged = merge_sel_results(results)
 
@@ -1101,7 +1101,7 @@ def _masked_result_drop_slice(key, data=None):
         if isinstance(k, np.ndarray):
             if is_duck_dask_array(data):
                 new_keys.append(_dask_array_with_chunks_hint(k, chunks_hint))
-            elif isinstance(data, sparse_array_type):
+            elif isinstance(data, array_type("sparse")):
                 import sparse
 
                 new_keys.append(sparse.COO.from_numpy(k))
@@ -1381,7 +1381,7 @@ class DaskIndexingAdapter(ExplicitlyIndexedNDArrayMixin):
                 return value
 
     def __setitem__(self, key, value):
-        if dask_version >= Version("2021.04.1"):
+        if mod_version("dask") >= Version("2021.04.1"):
             if isinstance(key, BasicIndexer):
                 self.array[key.tuple] = value
             elif isinstance(key, VectorizedIndexer):
@@ -1422,7 +1422,7 @@ class PandasIndexingAdapter(ExplicitlyIndexedNDArrayMixin):
         if dtype is None:
             self._dtype = get_valid_numpy_dtype(array)
         else:
-            self._dtype = np.dtype(dtype)  # type: ignore[assignment]
+            self._dtype = np.dtype(dtype)
 
     @property
     def dtype(self) -> np.dtype:
