@@ -207,7 +207,7 @@ class Weighted(Generic[T_Xarray]):
     def _reduce(
         da: DataArray,
         weights: DataArray,
-        dim: Dims | ellipsis = None,
+        dim: Dims = None,
         skipna: bool | None = None,
     ) -> DataArray:
         """reduce using dot; equivalent to (da * weights).sum(dim, skipna)
@@ -312,7 +312,7 @@ class Weighted(Generic[T_Xarray]):
         da: DataArray,
         q: ArrayLike,
         dim: Dims = None,
-        skipna: bool = None,
+        skipna: bool | None = None,
     ) -> DataArray:
         """Apply a weighted ``quantile`` to a DataArray along some dimension(s)."""
 
@@ -516,7 +516,7 @@ class Weighted(Generic[T_Xarray]):
         q: ArrayLike,
         *,
         dim: Dims = None,
-        keep_attrs: bool = None,
+        keep_attrs: bool | None = None,
         skipna: bool = True,
     ) -> T_Xarray:
 
