@@ -3,18 +3,28 @@ from __future__ import annotations
 import functools
 import itertools
 import warnings
-from typing import TYPE_CHECKING, Any, Callable, Hashable, Iterable, Literal
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Generic,
+    Hashable,
+    Iterable,
+    Literal,
+    TypeVar,
+    cast,
+)
 
 import numpy as np
 
 from ..core.formatting import format_item
+from ..core.types import HueStyleOptions, T_Xarray
 from .utils import (
     _LINEWIDTH_RANGE,
     _MARKERSIZE_RANGE,
     _add_legend,
     _determine_guide,
     _get_nice_quiver_magnitude,
-    _infer_meta_data,
     _infer_xy_labels,
     _Normalize,
     _parse_size,
@@ -33,7 +43,7 @@ if TYPE_CHECKING:
     from matplotlib.text import Annotation
 
     from ..core.dataarray import DataArray
-    from ..core.types import HueStyleOptions, Self
+
 
 # Overrides axes.labelsize, xtick.major.size, ytick.major.size
 # from mpl.rcParams
@@ -55,7 +65,10 @@ def _nicetitle(coord, value, maxchar, template):
     return title
 
 
-class FacetGrid:
+T_FacetGrid = TypeVar("T_FacetGrid", bound="FacetGrid")
+
+
+class FacetGrid(Generic[T_Xarray]):
     """
     Initialize the Matplotlib figure and FacetGrid object.
 
@@ -81,7 +94,7 @@ class FacetGrid:
 
     Attributes
     ----------
-    axes : ndarray of matplotlib.axes.Axes
+    axs : ndarray of matplotlib.axes.Axes
         Array containing axes in corresponding position, as returned from
         :py:func:`matplotlib.pyplot.subplots`.
     col_labels : list of matplotlib.text.Annotation
@@ -96,10 +109,10 @@ class FacetGrid:
         sometimes the rightmost grid positions in the bottom row.
     """
 
-    data: DataArray
+    data: T_Xarray
     name_dicts: np.ndarray
     fig: Figure
-    axes: np.ndarray
+    axs: np.ndarray
     row_names: list[np.ndarray]
     col_names: list[np.ndarray]
     figlegend: Legend | None
@@ -121,7 +134,7 @@ class FacetGrid:
 
     def __init__(
         self,
-        data: DataArray,
+        data: T_Xarray,
         col: Hashable | None = None,
         row: Hashable | None = None,
         col_wrap: int | None = None,
@@ -135,8 +148,8 @@ class FacetGrid:
         """
         Parameters
         ----------
-        data : DataArray
-            xarray DataArray to be plotted.
+        data : DataArray or Dataset
+            DataArray or Dataset to be plotted.
         row, col : str
             Dimension names that define subsets of the data, which will be drawn
             on separate facets in the grid.
@@ -210,7 +223,7 @@ class FacetGrid:
             cbar_space = 1
             figsize = (ncol * size * aspect + cbar_space, nrow * size)
 
-        fig, axes = plt.subplots(
+        fig, axs = plt.subplots(
             nrow,
             ncol,
             sharex=sharex,
@@ -245,7 +258,7 @@ class FacetGrid:
         self.data = data
         self.name_dicts = name_dicts
         self.fig = fig
-        self.axes = axes
+        self.axs = axs
         self.row_names = row_names
         self.col_names = col_names
 
@@ -269,17 +282,45 @@ class FacetGrid:
         self._mappables = []
         self._finalized = False
 
+    @property
+    def axes(self) -> np.ndarray:
+        warnings.warn(
+            (
+                "self.axes is deprecated since 2022.11 in order to align with "
+                "matplotlibs plt.subplots, use self.axs instead."
+            ),
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self.axs
+
+    @axes.setter
+    def axes(self, axs: np.ndarray) -> None:
+        warnings.warn(
+            (
+                "self.axes is deprecated since 2022.11 in order to align with "
+                "matplotlibs plt.subplots, use self.axs instead."
+            ),
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        self.axs = axs
+
     @property
     def _left_axes(self) -> np.ndarray:
-        return self.axes[:, 0]
+        return self.axs[:, 0]
 
     @property
     def _bottom_axes(self) -> np.ndarray:
-        return self.axes[-1, :]
+        return self.axs[-1, :]
 
     def map_dataarray(
-        self, func: Callable, x: Hashable | None, y: Hashable | None, **kwargs: Any
-    ) -> FacetGrid:
+        self: T_FacetGrid,
+        func: Callable,
+        x: Hashable | None,
+        y: Hashable | None,
+        **kwargs: Any,
+    ) -> T_FacetGrid:
         """
         Apply a plotting function to a 2d facet's subset of the data.
 
@@ -330,7 +371,7 @@ class FacetGrid:
             rgb=kwargs.get("rgb", None),
         )
 
-        for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+        for d, ax in zip(self.name_dicts.flat, self.axs.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
@@ -347,8 +388,12 @@ class FacetGrid:
         return self
 
     def map_plot1d(
-        self, func: Callable, x: Hashable, y: Hashable, **kwargs: Any
-    ) -> Self:
+        self: T_FacetGrid,
+        func: Callable,
+        x: Hashable | None,
+        y: Hashable | None,
+        **kwargs: Any,
+    ) -> T_FacetGrid:
         """
         Apply a plotting function to a 1d facet's subset of the data.
 
@@ -385,18 +430,24 @@ class FacetGrid:
         hueplt_norm = _Normalize(hueplt)
         self._hue_var = hueplt
         cbar_kwargs = kwargs.pop("cbar_kwargs", {})
-        if not hueplt_norm.data_is_numeric:
-            # TODO: Ticks seems a little too hardcoded, since it will always
-            # show all the values. But maybe it's ok, since plotting hundreds
-            # of categorical data isn't that meaningful anyway.
-            cbar_kwargs.update(format=hueplt_norm.format, ticks=hueplt_norm.ticks)
-            kwargs.update(levels=hueplt_norm.levels)
-        if "label" not in cbar_kwargs:
-            cbar_kwargs["label"] = label_from_attrs(hueplt_norm.data)
-        cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
-            func, hueplt_norm.values.to_numpy(), cbar_kwargs=cbar_kwargs, **kwargs
-        )
-        self._cmap_extend = cmap_params.get("extend")
+
+        if hueplt_norm.data is not None:
+            if not hueplt_norm.data_is_numeric:
+                # TODO: Ticks seems a little too hardcoded, since it will always
+                # show all the values. But maybe it's ok, since plotting hundreds
+                # of categorical data isn't that meaningful anyway.
+                cbar_kwargs.update(format=hueplt_norm.format, ticks=hueplt_norm.ticks)
+                kwargs.update(levels=hueplt_norm.levels)
+
+            cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
+                func,
+                cast("DataArray", hueplt_norm.values).data,
+                cbar_kwargs=cbar_kwargs,
+                **kwargs,
+            )
+            self._cmap_extend = cmap_params.get("extend")
+        else:
+            cmap_params = {}
 
         # Handle sizes:
         _size_r = _MARKERSIZE_RANGE if func.__name__ == "scatter" else _LINEWIDTH_RANGE
@@ -404,7 +455,7 @@ class FacetGrid:
             size = kwargs.get(_size, None)
 
             sizeplt = self.data[size] if size else None
-            sizeplt_norm = _Normalize(sizeplt, _size_r)
+            sizeplt_norm = _Normalize(data=sizeplt, width=_size_r)
             if size:
                 self.data[size] = sizeplt_norm.values
                 kwargs.update(**{_size: size})
@@ -422,7 +473,7 @@ class FacetGrid:
         func_kwargs["add_legend"] = False
         func_kwargs["add_title"] = False
 
-        add_labels_ = np.zeros(self.axes.shape + (3,), dtype=bool)
+        add_labels_ = np.zeros(self.axs.shape + (3,), dtype=bool)
         if kwargs.get("z") is not None:
             # 3d plots looks better with all labels. 3d plots can't sharex either so it
             # is easy to get lost while rotating the plots:
@@ -451,7 +502,7 @@ class FacetGrid:
 
         # Plot the data for each subplot:
         for add_lbls, d, ax in zip(
-            add_labels_.reshape((self.axes.size, -1)), name_dicts.flat, self.axes.flat
+            add_labels_.reshape((self.axs.size, -1)), name_dicts.flat, self.axs.flat
         ):
             func_kwargs["add_labels"] = add_lbls
             # None is the sentinel value
@@ -496,12 +547,15 @@ class FacetGrid:
 
         if add_colorbar:
             # Colorbar is after legend so it correctly fits the plot:
+            if "label" not in cbar_kwargs:
+                cbar_kwargs["label"] = label_from_attrs(hueplt_norm.data)
+
             self.add_colorbar(**cbar_kwargs)
 
         return self
 
     def map_dataarray_line(
-        self,
+        self: T_FacetGrid,
         func: Callable,
         x: Hashable | None,
         y: Hashable | None,
@@ -509,10 +563,10 @@ class FacetGrid:
         add_legend: bool = True,
         _labels=None,
         **kwargs: Any,
-    ) -> FacetGrid:
-        from .plot import _infer_line_data
+    ) -> T_FacetGrid:
+        from .dataarray_plot import _infer_line_data
 
-        for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+        for d, ax in zip(self.name_dicts.flat, self.axs.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
@@ -543,7 +597,7 @@ class FacetGrid:
         return self
 
     def map_dataset(
-        self,
+        self: T_FacetGrid,
         func: Callable,
         x: Hashable | None = None,
         y: Hashable | None = None,
@@ -551,7 +605,8 @@ class FacetGrid:
         hue_style: HueStyleOptions = None,
         add_guide: bool | None = None,
         **kwargs: Any,
-    ) -> FacetGrid:
+    ) -> T_FacetGrid:
+        from .dataset_plot import _infer_meta_data
 
         kwargs["add_guide"] = False
 
@@ -578,7 +633,7 @@ class FacetGrid:
             raise ValueError("Please provide scale.")
             # TODO: come up with an algorithm for reasonable scale choice
 
-        for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+        for d, ax in zip(self.name_dicts.flat, self.axs.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
@@ -612,7 +667,7 @@ class FacetGrid:
             self.set_titles()
             self.fig.tight_layout()
 
-            for ax, namedict in zip(self.axes.flat, self.name_dicts.flat):
+            for ax, namedict in zip(self.axs.flat, self.name_dicts.flat):
                 if namedict is None:
                     ax.set_visible(False)
 
@@ -672,7 +727,7 @@ class FacetGrid:
         if "label" not in kwargs:
             kwargs.setdefault("label", label_from_attrs(self.data))
         self.cbar = self.fig.colorbar(
-            self._mappables[-1], ax=list(self.axes.flat), **kwargs
+            self._mappables[-1], ax=list(self.axs.flat), **kwargs
         )
 
     def add_quiverkey(self, u: Hashable, v: Hashable, **kwargs: Any) -> None:
@@ -680,7 +735,7 @@ class FacetGrid:
 
         magnitude = _get_nice_quiver_magnitude(self.data[u], self.data[v])
         units = self.data[u].attrs.get("units", "")
-        self.quiverkey = self.axes.flat[-1].quiverkey(
+        self.quiverkey = self.axs.flat[-1].quiverkey(
             self._mappables[-1],
             X=0.8,
             Y=0.9,
@@ -706,7 +761,7 @@ class FacetGrid:
         Examples
         --------
         >>> ds = xr.tutorial.scatter_example_dataset(seed=42)
-        >>> fg = ds.plot.scatter("A", "B", hue="y", row="x", col="w")
+        >>> fg = ds.plot.scatter(x="A", y="B", hue="y", row="x", col="w")
         >>> round(fg._get_largest_lims()["x"][0], 3)
         -0.334
         """
@@ -716,7 +771,7 @@ class FacetGrid:
         for axis in ("x", "y", "z"):
             # Find the plot with the largest xlim values:
             lower, upper = lims_largest[axis]
-            for ax in self.axes.flat:
+            for ax in self.axs.flat:
                 get_lim: None | Callable[[], tuple[float, float]] = getattr(
                     ax, f"get_{axis}lim", None
                 )
@@ -748,15 +803,15 @@ class FacetGrid:
         Examples
         --------
         >>> ds = xr.tutorial.scatter_example_dataset(seed=42)
-        >>> fg = ds.plot.scatter("A", "B", hue="y", row="x", col="w")
+        >>> fg = ds.plot.scatter(x="A", y="B", hue="y", row="x", col="w")
         >>> fg._set_lims(x=(-0.3, 0.3), y=(0, 2), z=(0, 4))
-        >>> fg.axes[0, 0].get_xlim(), fg.axes[0, 0].get_ylim()
+        >>> fg.axs[0, 0].get_xlim(), fg.axs[0, 0].get_ylim()
         ((-0.3, 0.3), (0.0, 2.0))
         """
         lims_largest = self._get_largest_lims()
 
         # Set limits:
-        for ax in self.axes.flat:
+        for ax in self.axs.flat:
             for (axis, data_limit), parameter_limit in zip(
                 lims_largest.items(), (x, y, z)
             ):
@@ -827,7 +882,7 @@ class FacetGrid:
         nicetitle = functools.partial(_nicetitle, maxchar=maxchar, template=template)
 
         if self._single_group:
-            for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+            for d, ax in zip(self.name_dicts.flat, self.axs.flat):
                 # Only label the ones with data
                 if d is not None:
                     coord, value = list(d.items()).pop()
@@ -836,7 +891,7 @@ class FacetGrid:
         else:
             # The row titles on the right edge of the grid
             for index, (ax, row_name, handle) in enumerate(
-                zip(self.axes[:, -1], self.row_names, self.row_labels)
+                zip(self.axs[:, -1], self.row_names, self.row_labels)
             ):
                 title = nicetitle(coord=self._row_var, value=row_name, maxchar=maxchar)
                 if not handle:
@@ -855,7 +910,7 @@ class FacetGrid:
 
             # The column titles on the top row
             for index, (ax, col_name, handle) in enumerate(
-                zip(self.axes[0, :], self.col_names, self.col_labels)
+                zip(self.axs[0, :], self.col_names, self.col_labels)
             ):
                 title = nicetitle(coord=self._col_var, value=col_name, maxchar=maxchar)
                 if not handle:
@@ -891,7 +946,7 @@ class FacetGrid:
         x_major_locator = MaxNLocator(nbins=max_xticks)
         y_major_locator = MaxNLocator(nbins=max_yticks)
 
-        for ax in self.axes.flat:
+        for ax in self.axs.flat:
             ax.xaxis.set_major_locator(x_major_locator)
             ax.yaxis.set_major_locator(y_major_locator)
             for tick in itertools.chain(
@@ -899,7 +954,9 @@ class FacetGrid:
             ):
                 tick.label1.set_fontsize(fontsize)
 
-    def map(self, func: Callable, *args: Any, **kwargs: Any) -> FacetGrid:
+    def map(
+        self: T_FacetGrid, func: Callable, *args: Hashable, **kwargs: Any
+    ) -> T_FacetGrid:
         """
         Apply a plotting function to each facet's subset of the data.
 
@@ -910,7 +967,7 @@ class FacetGrid:
             must plot to the currently active matplotlib Axes and take a
             `color` keyword argument. If faceting on the `hue` dimension,
             it must also take a `label` keyword argument.
-        *args : strings
+        *args : Hashable
             Column names in self.data that identify variables with data to
             plot. The data for each variable is passed to `func` in the
             order the variables are specified in the call.
@@ -924,7 +981,7 @@ class FacetGrid:
         """
         plt = import_matplotlib_pyplot()
 
-        for ax, namedict in zip(self.axes.flat, self.name_dicts.flat):
+        for ax, namedict in zip(self.axs.flat, self.name_dicts.flat):
             if namedict is not None:
                 data = self.data.loc[namedict]
                 plt.sca(ax)
@@ -941,7 +998,7 @@ class FacetGrid:
 
 
 def _easy_facetgrid(
-    data: DataArray,
+    data: T_Xarray,
     plotfunc: Callable,
     kind: Literal["line", "dataarray", "dataset", "plot1d"],
     x: Hashable | None = None,
@@ -957,7 +1014,7 @@ def _easy_facetgrid(
     ax: Axes | None = None,
     figsize: Iterable[float] | None = None,
     **kwargs: Any,
-) -> FacetGrid:
+) -> FacetGrid[T_Xarray]:
     """
     Convenience method to call xarray.plot.FacetGrid from 2d plotting methods
 
@@ -1001,4 +1058,6 @@ def _easy_facetgrid(
     if kind == "dataset":
         return g.map_dataset(plotfunc, x, y, **kwargs)
 
-    raise ValueError(f"kind must be one of `line`, `dataarray`, `dataset`, got {kind}")
+    raise ValueError(
+        f"kind must be one of `line`, `dataarray`, `dataset` or `plot1d`, got {kind}"
+    )
